@@ -1,25 +1,15 @@
 import typer
 
-from ..build_env import get_build_environment_vars, get_pyodide_root, init_environment
+from pyodide_build.build_env import (
+    get_build_environment_vars,
+    get_pyodide_root,
+    init_environment,
+)
+from pyodide_build.config import PYODIDE_CLI_CONFIGS
 
 app = typer.Typer(help="Manage config variables used in pyodide")
 
 
-# A dictionary of config variables {key: env_var_in_makefile}
-PYODIDE_CONFIGS = {
-    "emscripten_version": "PYODIDE_EMSCRIPTEN_VERSION",
-    "python_version": "PYVERSION",
-    "rustflags": "RUSTFLAGS",
-    "cmake_toolchain_file": "CMAKE_TOOLCHAIN_FILE",
-    "pyo3_config_file": "PYO3_CONFIG_FILE",
-    "rust_toolchain": "RUST_TOOLCHAIN",
-    "cflags": "SIDE_MODULE_CFLAGS",
-    "cxxflags": "SIDE_MODULE_CXXFLAGS",
-    "ldflags": "SIDE_MODULE_LDFLAGS",
-    "meson_cross_file": "MESON_CROSS_FILE",
-}
-
-
 @app.callback(no_args_is_help=True)
 def callback() -> None:
     return
@@ -30,7 +20,7 @@ def _get_configs() -> dict[str, str]:
 
     configs: dict[str, str] = get_build_environment_vars(get_pyodide_root())
 
-    configs_filtered = {k: configs[v] for k, v in PYODIDE_CONFIGS.items()}
+    configs_filtered = {k: configs[v] for k, v in PYODIDE_CLI_CONFIGS.items()}
     return configs_filtered
 
 
@@ -42,7 +32,7 @@ def list_config():
     configs = _get_configs()
 
     for k, v in configs.items():
-        typer.echo(f"{k}={v}")
+        typer.echo(f'{k}="{v}"')
 
 
 @app.command("get")
