@@ -77,6 +77,9 @@ class CrossCompileArgs(NamedTuple):
     target_install_dir: str = ""  # The path to the target Python installation
     pythoninclude: str = ""  # path to the cross-compiled Python include directory
     exports: Literal["whole_archive", "requested", "pyinit"] | list[str] = "pyinit"
+    # Pyodide abi, e.g., 2025_0
+    # Sometimes we have to inject compile flags only for certain abis.
+    abi: str = ""
 
 
 def is_link_cmd(line: list[str]) -> bool:
@@ -93,7 +96,7 @@ def is_link_cmd(line: list[str]) -> bool:
     return False
 
 
-def replay_genargs_handle_dashl(arg: str, used_libs: set[str]) -> str | None:
+def replay_genargs_handle_dashl(arg: str, used_libs: set[str], abi: str) -> str | None:
     """
     Figure out how to replace a `-lsomelib` argument.
 
@@ -118,6 +121,12 @@ def replay_genargs_handle_dashl(arg: str, used_libs: set[str]) -> str | None:
     if arg == "-lgfortran":
         return None
 
+    # Some Emscripten libraries that use setjmp/longjmp.
+    # The Emscripten linker should automatically know to use these variants so
+    # this shouldn't be necessary.
+    if abi > "2025" and arg in ["-lfreetype", "-lpng"]:
+        arg += "-legacysjlj"
+
     # WASM link doesn't like libraries being included twice
     # skip second one
     if arg in used_libs:
@@ -339,19 +348,32 @@ def _calculate_object_exports_readobj_parse(output: str) -> list[str]:
 def calculate_object_exports_readobj(objects: list[str]) -> list[str] | None:
     import shutil
 
-    readobj_path = shutil.which("llvm-readobj")
+    # This works for bootstrapped Emscripten via GitHub sources.
+    # llvm-readobj might not be available this way with Homebrew
+    # or conda-forge distributions of Emscripten.
+    which_emcc = shutil.which("emcc")
+    assert which_emcc
+    emcc = Path(which_emcc)
+    readobj = (emcc / "../../bin/llvm-readobj").resolve()
+    if readobj.exists():
+        readobj_path = str(readobj)
+    else:
+        readobj_path = shutil.which("llvm-readobj")
     if not readobj_path:
-        which_emcc = shutil.which("emcc")
-        assert which_emcc
-        emcc = Path(which_emcc)
-        readobj_path = str((emcc / "../../bin/llvm-readobj").resolve())
+        print("Failed to find llvm-readobj, quitting", file=sys.stdout)
+        sys.exit(1)
+
     args = [
         readobj_path,
         "--section-details",
         "-st",
     ] + objects
     completedprocess = subprocess.run(
-        args, encoding="utf8", capture_output=True, env={"PATH": os.environ["PATH"]}
+        args,
+        encoding="utf8",
+        capture_output=True,
+        env={"PATH": os.environ["PATH"]},
+        check=False,
     )
     if completedprocess.returncode:
         print(f"Command '{' '.join(args)}' failed. Output to stderr was:")
@@ -367,7 +389,11 @@ def calculate_object_exports_readobj(objects: list[str]) -> list[str] | None:
 def calculate_object_exports_nm(objects: list[str]) -> list[str]:
     args = ["emnm", "-j", "--export-symbols"] + objects
     result = subprocess.run(
-        args, encoding="utf8", capture_output=True, env={"PATH": os.environ["PATH"]}
+        args,
+        encoding="utf8",
+        capture_output=True,
+        env={"PATH": os.environ["PATH"]},
+        check=False,
     )
     if result.returncode:
         print(f"Command '{' '.join(args)}' failed. Output to stderr was:")
@@ -475,9 +501,9 @@ def handle_command_generate_args(  # noqa: C901
         return ["emcc", "-v"]
 
     cmd = line[0]
-    if cmd == "c++" or cmd == "g++":
+    if cmd in {"c++", "g++"}:
         new_args = ["em++"]
-    elif cmd in ("cc", "gcc", "ld", "lld"):
+    elif cmd in {"cc", "gcc", "ld", "lld"}:
         new_args = ["emcc"]
         # distutils doesn't use the c++ compiler when compiling c++ <sigh>
         if any(arg.endswith((".cpp", ".cc")) for arg in line):
@@ -495,10 +521,6 @@ def handle_command_generate_args(  # noqa: C901
             "emcmake",
             "cmake",
             *flags,
-            # Since we create a temporary directory and install compiler symlinks every time,
-            # CMakeCache.txt will contain invalid paths to the compiler when re-running,
-            # so we need to tell CMake to ignore the existing cache and build from scratch.
-            "--fresh",
         ]
         return line
     elif cmd == "meson":
@@ -514,7 +536,7 @@ def handle_command_generate_args(  # noqa: C901
             ]
 
         return line
-    elif cmd in ("install_name_tool", "otool"):
+    elif cmd in {"install_name_tool", "otool"}:
         # In MacOS, meson tries to run install_name_tool to fix the rpath of the shared library
         # assuming that it is a ELF file. We need to skip this step.
         # See: https://github.com/mesonbuild/meson/issues/8027
@@ -538,7 +560,7 @@ def handle_command_generate_args(  # noqa: C901
             continue
 
         if arg.startswith("-l"):
-            result = replay_genargs_handle_dashl(arg, used_libs)
+            result = replay_genargs_handle_dashl(arg, used_libs, build_args.abi)
         elif arg.startswith("-I"):
             result = replay_genargs_handle_dashI(arg, build_args.target_install_dir)
         elif arg.startswith("-Wl"):
@@ -591,24 +613,24 @@ def handle_command(
     """
 
     if line[0] == "gfortran":
-        from pyodide_build._f2c_fixes import replay_f2c
+        from _f2c_fixes import replay_f2c
 
         tmp = replay_f2c(line)
         if tmp is None:
             # No source file, it's a query for information about the compiler. Pretend we're
             # gfortran by letting gfortran handle it
-            return subprocess.run(line).returncode
+            return subprocess.run(line, check=False).returncode
 
         line = tmp
 
     new_args = handle_command_generate_args(line, build_args)
 
     if build_args.pkgname == "scipy":
-        from pyodide_build._f2c_fixes import scipy_fixes
+        from _f2c_fixes import scipy_fixes
 
         scipy_fixes(new_args)
 
-    result = subprocess.run(new_args)
+    result = subprocess.run(new_args, check=False)
     return result.returncode
 
 
@@ -621,6 +643,7 @@ def compiler_main():
         target_install_dir=PYWASMCROSS_ARGS["target_install_dir"],
         pythoninclude=PYWASMCROSS_ARGS["pythoninclude"],
         exports=PYWASMCROSS_ARGS["exports"],
+        abi=PYWASMCROSS_ARGS["abi"],
     )
     basename = Path(sys.argv[0]).name
     args = list(sys.argv)
